from datetime import datetime
from peewee import fn
from .db import Task, Category, TaskStatus, db


def add_task(
    text: str,
    category: str,
    status: str = TaskStatus.ZERO.value,
    due: datetime | None = None,
    type: str | None = None,
) -> Task:
    """
    Add a new task to the database.
    Returns the created task instance.
    """
    with db.atomic():
        category_id = None
        if category:
            category, _ = Category.get_or_create(name=category)
            category_id = category.id
            task = Task.create(
                text=text, type=type, status=status, due=due, category_id=category_id
            )
    return task


def update_task(
    task_id: int,
    **kwargs,
) -> Task:
    with db.atomic():
        task = Task.get_by_id(task_id)

        query = Task.update(kwargs).where(Task.id == task_id)
        query.execute()
        task = Task.get_by_id(task_id)
    return task


def get_tasks(
    search_text: str | None = None,
    category: int | None = None,
    status: str | None = None,
    include_done: bool = False,
) -> list[Task]:
    query = Task.select()

    if search_text:
        query = query.where(fn.Lower(Task.text).contains(search_text.lower()))
    if category:
        query = query.where(Task.category == Category.get(name=category))
    if status:
        query = query.where(Task.status == status)

    if not include_done:
        # by default, exclude done tasks
        query = query.where(Task.status != TaskStatus.DONE.value)

    # order by due date (null last) and created date
    query = query.order_by(
        fn.COALESCE(Task.due, datetime(3000, 12, 31)), Task.created_at
    )

    return list(query)


def get_categories() -> list[Category]:
    return list(Category.select().order_by(Category.name))
